//=============================================================================
//
//  CLASS Raytracer
//
//=============================================================================
#ifndef RAYTRACER_WHITTED_H
#define RAYTRACER_WHITTED_H

//== INCLUDES =================================================================
#include <list>
#include <qimage.h>
#include <qprogressdialog.h>
#include "Types.h"
#include "PinholeCamera.h"
#include "Light.h"
#include "Surface.h"
#include "SurfaceList.h"

typedef std::list<Light *>  LightListType;

//== CLASS DEFINITION =========================================================

/*!
 *  Whitted-style raytracer
 *
 *	
 */
class Raytracer : public QObject
{
    Q_OBJECT

    //-------------------------------------------------------------- private data
private:
    PinholeCamera   m_clPinholeCamera;
    ColorType       m_colBackground;
    ColorType       m_colAmbient;
    bool            m_bOwnsLights;

    //-------------------------------------------------------------- private methods
private:
    ColorType   traceRay( const Surface * pclScene,
                          const Ray & clRay,
                          unsigned int uiDepth,
                          RayArray * glRays );
    ColorType   shade( const Surface *pclScene, TracingContext &tContext );

    //-------------------------------------------------------------- constructors
public:

    //! default constructor
    Raytracer() :
    m_colBackground( 0 ),
    m_colAmbient( 0 ),
    m_bOwnsLights( true )
    {
    };

    //! destructor
    ~Raytracer()
    {
        removeAllLights();
    };
    LightListType   m_LightList;

    //----------------------------------------------------------- element access
public:

    //@{

    //!Background color
    void setBackgroundColor(
        const ColorType &colBackground )    
    {
        m_colBackground = colBackground;
    };

    
    const ColorType &getBackgroundColor() const
    {
        return m_colBackground;
    };

    //!Manage lights
    void AddLight(
        Light   *pclLight ) 
    {
        m_LightList.push_back( pclLight );
    };
    void    removeAllLights();

    //@}

    //!Manage camera
    PinholeCamera &getCamera()
    {
        return m_clPinholeCamera;
    };

    //----------------------------------------------------------- functionality
    //! render scene with the classic algorithm
    void    render( SurfaceList   *pclScene,
                    unsigned int    uiMaxDepth,
                    QImage          *clQImage,
                    RenderingMode   mode = MODE_TIMING );

    //----------------------------------------------------------- GL/Qt Stuff
    //render Lights in OpenGL
    void    renderLightsGL();
signals:
    void    renderImageAndRays( RayArray * glRays );
};

//=============================================================================
#endif // RAYTRACER_WHITTED_H

