//=============================================================================
//
//  CLASS Shader
//
//=============================================================================
#ifndef RAYTRACER_SHADER_H
#define RAYTRACER_SHADER_H

//== INCLUDES =================================================================
#include "Types.h"
#include "Ray.h"

//== CLASS DEFINITION =========================================================

/*!
	Abstract base class for shaders
*/
class   Shader
{

    //-------------------------------------------------------------- protected data
protected:
    ColorType   m_colAmbient;
    ColorType   m_colSpecular;
    ColorType   m_colRefractive;
    double    m_rRefractiveIndex;

    //-------------------------------------------------------------- private methods
private:

public:

    //-------------------------------------------------------------- constructors

    //! initializing constructor
    Shader(
        const ColorType &colAmbient = ColorType(0),     
        const ColorType &colSpecular = ColorType(0),    
        const ColorType &colRefractive = ColorType(0),  
        const double  rRefractiveIndex = 1 )  :
    m_colAmbient( colAmbient ),
    m_colSpecular( colSpecular ),
    m_colRefractive( colRefractive ),
    m_rRefractiveIndex( rRefractiveIndex )
    {
    };

    //! destructor	
    ~Shader()
    {
    };

    //----------------------------------------------------------- element access
public:
    
    void setAmbientColor(
        const ColorType &colAmbient )   
    {
        m_colAmbient = colAmbient;
    }

    
    const ColorType &getAmbientColor() const
    {
        return m_colAmbient;
    }

    
    void setSpecularColor(
        const ColorType &colSpecular )  
    {
        m_colSpecular = colSpecular;
    }

    
    const ColorType &getSpecularColor() const
    {
        return m_colSpecular;
    }

    
    void setRefractiveColor(
        const ColorType &colRefractive )    
    {
        m_colRefractive = colRefractive;
    };

    
    const ColorType &getRefractiveColor() const
    {
        return m_colRefractive;
    };

    
    void setRefractiveIndex(
        const double  rRefractiveIndex )  
    {
        m_rRefractiveIndex = rRefractiveIndex;
    }

    
    const double getRefractiveIndex() const
    {
        return m_rRefractiveIndex;
    };

    //----------------------------------------------------------- class info
public:
    
    bool isSpecular() const
    {
        return ( m_colSpecular.getEnergy() > (double) 0.0 );
    };

    
    bool isRefractive() const
    {
		if ( m_rRefractiveIndex < 1 - EPSILON || m_rRefractiveIndex > 1 + EPSILON )
			return true;
		return false;
    };

    //----------------------------------------------------------- functionality
public:

    /*!
		shade.
		evaluate the BRDF times Light weighted by the cosine of the incoming direction
	*/
    virtual ColorType   shade( const TracingContext &tContext ) const = 0;

    /* shade in OpenGLMode */
    virtual void        shadeGL() const = 0;

    /*!
		evaluate ambient term for Whitted-Style ray tracing
	*/
    virtual ColorType shadeAmbient(
        const ColorType &colAmbient )  const
    {
        return colAmbient * m_colAmbient;
    };
};
//=============================================================================
#endif // RAYTRACER_SHADER_H
