//=============================================================================
//
//  CLASS SuperEllipsoid
//
//=============================================================================
#ifndef RAYTRACER_SUPERELLIPSOID_H
#define RAYTRACER_SUPERELLIPSOID_H

//== INCLUDES =================================================================
#include "Surface.h"


//== CLASS DEFINITION =========================================================

/*!
	A SuperEllipsoid. See Lecture for Details!
*/
class SuperEllipsoid : public Surface
{

    //-------------------------------------------------------------- private data
private:
    VectorType3 m_v3Position;
    double    m_a;
	double    m_b;
	double	m_c;
    double    m_p;
	double    m_q;
	double	m_r;



public:

    //-------------------------------------------------------------- constructors

    //! standard constructor
    SuperEllipsoid()
    {
    };

    //! initializing constructor
    SuperEllipsoid(
        const VectorType3   &v3Position,  
        const double      &a,
		const double      &b,
        const double      &c,
		const double      &p,
		const double      &q,
		const double      &r)  :
    m_v3Position( v3Position ),
    m_a( a ),
	m_b( b ),
	m_c( c ),
	m_p( p ),
	m_q( q ),
	m_r( r )
    {
    };

    //-------------------------------------------------------------- element access
    void setPosition(
        const VectorType3   &v3Position ) 
    {
        m_v3Position = v3Position;
    };

    
    const VectorType3 &getPosition() const
    {
        return m_v3Position;
    };

    
    void setDivisors(
        const double  &a,
		const double  &b,
		const double  &c )  
    {
        m_a = a;
        m_b = b;
        m_c = c;
    };

    
    void getDivisors(
        double  *a,
		double  *b,
		double  *c )
    {
        *a =  m_a;
        *b =  m_b;
        *c =  m_c;
    };

	
    void setExponents(
        const double  &p,
		const double  &q,
		const double  &r )  
    {
        m_p = p;
        m_q = q;
        m_r = r;
    };

    
    void getExponents(
        double  *p,
		double  *q,
		double  *r )
    {
        *p =  m_p;
        *q =  m_q;
        *r =  m_r;
    };
    
    AABBox getAABB() const
    {
        VectorType3 v3_offset = VectorType3( 1, 1, 1 );
        return AABBox( m_v3Position - v3_offset, m_v3Position + v3_offset );
    };

    //render object in OpenGL
    void    renderGL() const;

    //translate object
    void    translate( float x, float y, float z );

    //rotate object
    void    rotate( float rotx, float roty, float rotz );

    //get Translation
    void getTranslation(
        float   *x,     
        float   *y,     
        float   *z )    
    {
        *x = m_v3Position[0];
        *y = m_v3Position[1];
        *z = m_v3Position[2];
    };

    //get Rotation
    void getRotation(
        int *rotx,  
        int *roty,  
        int *rotz ) 
    {
        *rotx = 0;
        *roty = 0;
        *rotz = 0;
    };

    //--------------------------------------------------------------- intersections
    bool    intersect( const Ray       &clRay,
                       const double  t0,
                       const double  t1,
                       TracingContext  &tContext ) const;
};

//=============================================================================
#endif // RAYTRACER_SUPERELLIPSOID_H

